@@ -415,13 +415,13 @@ class ABCSMC:
             to do the first population). Usually 0 or history.max_t + 1.
         """
         def get_initial_sum_stats():
-            population = self._get_initial_population(t)
+            population = self._get_initial_population(t-1)
             # only the accepted sum stats are available initially
             sum_stats = population.get_accepted_sum_stats()
             return sum_stats
 
         def _get_initial_population_with_distances():
-            population = self._get_initial_population(t)
+            population = self._get_initial_population(t-1)
 
             def distance_to_ground_truth(x, par):
                 return self.distance_function(x, self.x_0, t, par)
@@ -445,14 +445,13 @@ class ABCSMC:
             population = _get_initial_population_with_distances()
             records = []
             for particle in population.get_list():
-                for d in particle.accepted_distances:
-                    # we use dummy densities here, since only the quotient
-                    # is of interest
-                    records.append({
-                        'distance': d,
-                        'transition_pd_prev': 1.0,
-                        'transition_pd': 1.0,
-                        'accepted': True})
+                # we use dummy densities here, since only the quotient
+                #  is of interest
+                records.append({
+                    'distance': particle.distance,
+                    'transition_pd_prev': 1.0,
+                    'transition_pd': 1.0,
+                    'accepted': True})
             return records
 
         self.eps.initialize(
@@ -472,11 +471,15 @@ class ABCSMC:
         .. warning::
             The sample is cached. Thus, the function can be called repeatedly
             without further computational overhead.
+
+        Parameters
+        ----------
+        t: The time for which to draw samples, i.e. one before the target time.
         """
         if self._initial_population is None:
             if self.history.n_populations > 0:
                 # extract latest population from database
-                population = self.history.get_population()
+                population = self.history.get_population(t=t)
             else:
                 # sample
                 population = self._sample_from_prior(t)
@@ -484,13 +487,13 @@ class ABCSMC:
 
         return self._initial_population
 
-    def _create_simulate_from_prior_function(self, t: int):
+    def _create_simulate_from_prior_function(self):
         """
         Similar to _create_simulate_function, apart here we sample from the
         prior and accept all.
         """
         return create_simulate_from_prior_function(
-            t=t, model_prior=self.model_prior,
+            model_prior=self.model_prior,
             parameter_priors=self.parameter_priors, models=self.models,
             summary_statistics=self.summary_statistics,
         )
@@ -501,9 +504,9 @@ class ABCSMC:
         the history of the distance function or the epsilon.
         """
         # create simulate function
-        simulate_one = self._create_simulate_from_prior_function(t)
+        simulate_one = self._create_simulate_from_prior_function()
 
-        logger.info(f"Calibration sample before t={t}.")
+        logger.info(f"Calibration sample t = {t}.")
 
         # call sampler
         sample = self.sampler.sample_until_n_accepted(
@@ -542,14 +545,11 @@ class ABCSMC:
             happens. Therefore, the returned function should be light, and
             in particular not contain references to the ABCSMC class.
         """
-        nr_samples_per_parameter = \
-            self.population_size.nr_samples_per_parameter
         return create_simulate_function(
             t=t, model_probabilities=self.history.get_model_probabilities(t-1),
             model_perturbation_kernel=self.model_perturbation_kernel,
             transitions=self.transitions, model_prior=self.model_prior,
             parameter_priors=self.parameter_priors,
-            nr_samples_per_parameter=nr_samples_per_parameter,
             models=self.models, summary_statistics=self.summary_statistics,
             x_0=self.x_0, distance_function=self.distance_function,
             eps=self.eps, acceptor=self.acceptor,
@@ -662,7 +662,7 @@ class ABCSMC:
             if current_eps is None or np.isnan(current_eps):
                 raise ValueError(
                     f"The epsilon threshold {current_eps} is invalid.")
-            logger.info(f"t: {t}, eps: {current_eps}.")
+            logger.info(f"t: {t}, eps: {current_eps:.8e}.")
 
             # create simulate function
             simulate_one = self._create_simulate_function(t)
@@ -673,7 +673,7 @@ class ABCSMC:
                 else pop_size / min_acceptance_rate
 
             # perform the sampling
-            logger.debug(f"Now submitting population {t}.")
+            logger.debug(f"Submitting population {t}.")
             sample = self.sampler.sample_until_n_accepted(
                 n=pop_size, simulate_one=simulate_one, t=t,
                 max_eval=max_eval, ana_vars=self._vars(),
@@ -702,8 +702,8 @@ class ABCSMC:
             acceptance_rate = pop_size / n_sim
             ess = effective_sample_size(
                 population.get_weighted_distances()['w'])
-            logger.info(f"Acceptance rate: {pop_size} / {n_sim} = "
-                        f"{acceptance_rate:.4e}, ESS={ess:.4e}.")
+            logger.info(f"Accepted: {pop_size} / {n_sim} = "
+                        f"{acceptance_rate:.4e}, ESS: {ess:.4e}.")
 
             # prepare next iteration
             self._prepare_next_iteration(
@@ -792,20 +792,16 @@ class ABCSMC:
 
             # iterate over all particles
             for particle in recorded_particles:
-                all_distances = \
-                    particle.accepted_distances + particle.rejected_distances
-                # evaluate previous and currenttransition density
+                # evaluate previous and current transition density
                 transition_pd_prev = transition_pdf_prev(
                     particle.m, particle.parameter)
                 transition_pd = transition_pdf(
                     particle.m, particle.parameter)
-                # iterate over all distances
-                for d in all_distances:
-                    records.append({
-                        'distance': d,
-                        'transition_pd_prev': transition_pd_prev,
-                        'transition_pd': transition_pd,
-                        'accepted': particle.accepted})
+                records.append({
+                    'distance': particle.distance,
+                    'transition_pd_prev': transition_pd_prev,
+                    'transition_pd': transition_pd,
+                    'accepted': particle.accepted})
             return records
 
         # update epsilon
@@ -860,14 +856,11 @@ class ABCSMC:
         These variables are passed to the sampler, as some need to create
         simulation settings themselves.
         """
-        nr_samples_per_parameter = \
-            self.population_size.nr_samples_per_parameter
         return AnalysisVars(
             model_prior=self.model_prior,
             parameter_priors=self.parameter_priors,
             model_perturbation_kernel=self.model_perturbation_kernel,
             transitions=self.transitions,
-            nr_samples_per_parameter=nr_samples_per_parameter,
             models=self.models,
             summary_statistics=self.summary_statistics,
             x_0=self.x_0,
