@@ -4,13 +4,15 @@ from typing import Callable, Union
 import copy
 
 import pyabc
-from .base import PetabImporter
+from .base import PetabImporter, rescale
 
-logger = logging.getLogger(__name__)
+logger = logging.getLogger("ABC.PEtab")
 
 try:
     import petab
+    import petab.C as C
 except ImportError:
+    petab = C = None
     logger.error("Install petab (see https://github.com/icb-dcm/petab) to use "
                  "the petab functionality.")
 
@@ -19,6 +21,7 @@ try:
     import amici.petab_import
     from amici.petab_objective import simulate_petab, LLH, RDATAS
 except ImportError:
+    amici = amici.petab_import = simulate_petab = LLH = RDATAS = None
     logger.error("Install amici (see https://github.com/icb-dcm/amici) to use "
                  "the amici functionality.")
 
@@ -29,16 +32,9 @@ class AmiciPetabImporter(PetabImporter):
 
     Parameters
     ----------
-
     petab_problem:
         A PEtab problem containing all information on the parameter estimation
         problem.
-    free_parameters:
-        Whether to estimate free parameters (column ESTIMATE=1 in the
-        parameters table).
-    fixed_parameters:
-        Whether to estimate fixed parameters (column ESTIMATE=0 in the
-        parameters table).
     amici_model:
         A corresponding compiled AMICI model that allows simulating data for
         parameters. If not provided, one is created using
@@ -52,13 +48,8 @@ class AmiciPetabImporter(PetabImporter):
             self,
             petab_problem: petab.Problem,
             amici_model: amici.Model = None,
-            amici_solver: amici.Solver = None,
-            free_parameters: bool = True,
-            fixed_parameters: bool = False):
-        super().__init__(
-            petab_problem=petab_problem,
-            free_parameters=free_parameters,
-            fixed_parameters=fixed_parameters)
+            amici_solver: amici.Solver = None):
+        super().__init__(petab_problem=petab_problem)
 
         if amici_model is None:
             amici_model = amici.petab_import.import_petab_problem(
@@ -96,37 +87,52 @@ class AmiciPetabImporter(PetabImporter):
             The model returns already the likelihood value.
         """
         # parameter ids to consider
-        x_ids = self.petab_problem.get_x_ids(
-            free=self.free_parameters, fixed=self.fixed_parameters)
+        x_free_ids = self.petab_problem.get_x_ids(free=True, fixed=False)
 
-        # fixed paramters
+        # fixed parameters
         x_fixed_ids = self.petab_problem.get_x_ids(
-            free=not self.free_parameters, fixed=not self.fixed_parameters)
+            free=False, fixed=True)
         x_fixed_vals = self.petab_problem.get_x_nominal(
-            scaled=True,
-            free=not self.free_parameters, fixed=not self.fixed_parameters)
+            scaled=True, free=False, fixed=True)
 
         # extract variables for improved pickling
         petab_problem = self.petab_problem
         amici_model = self.amici_model
         amici_solver = self.amici_solver
+        prior_scales = self.prior_scales
+        scaled_scales = self.scaled_scales
+
+        if set(prior_scales.keys()) != set(x_free_ids):
+            # this should not happen
+            raise AssertionError("Parameter id mismatch")
 
         # no gradients for pyabc
         amici_solver.setSensitivityOrder(0)
 
         def model(par: Union[Sequence, Mapping]) -> Mapping:
-            """The model function."""
+            """The model function.
+
+            Note: The parameters are assumed to be passed on prior scale.
+            """
             # copy since we add fixed parameters
             par = copy.deepcopy(par)
 
             # convenience to allow calling model not only with dicts
             if not isinstance(par, Mapping):
-                par = {key: val for key, val in zip(x_ids, par)}
+                par = {key: val for key, val in zip(x_free_ids, par)}
 
             # add fixed parameters
             for key, val in zip(x_fixed_ids, x_fixed_vals):
                 par[key] = val
 
+            # scale parameters whose priors are not on scale
+            for key in prior_scales.keys():
+                par[key] = rescale(
+                    val=par[key],
+                    origin_scale=prior_scales,
+                    target_scale=scaled_scales,
+                )
+
             # simulate model
             sim = simulate_petab(
                 petab_problem=petab_problem,
