@@ -12,6 +12,13 @@ STATIC = "static"
 DYNAMIC = "dynamic"
 # the queue to return results through
 QUEUE = "queue"
+# collection of active indices
+ACTIVE_SET = "active_set"
+
+# lock for evaluation counters
+EVAL_LOCK = "eval_lock"
+# lock for accessing the active set data structure
+ACTIVE_SET_LOCK = "active_set_lock"
 
 # default sleep time
 SLEEP_TIME = .1
@@ -28,6 +35,8 @@ ALL_ACCEPTED = "all_accepted"
 SSA = "sample_simulate_accept"
 # whether look-ahead mode is to be employed
 IS_LOOK_AHEAD = "is_look_ahead"
+# maximum number of evaluations in look-ahead mode
+MAX_N_EVAL_LOOK_AHEAD = "max_n_eval_look_ahead"
 # batch size to use
 BATCH_SIZE = "batch_size"
 
