@@ -1,10 +1,9 @@
 """Redis based sampler base class and dynamic scheduling samplers."""
 
 import numpy as np
-import pickle
 from time import sleep
 from datetime import datetime
-import cloudpickle
+import cloudpickle as pickle
 import copy
 import logging
 from redis import StrictRedis
@@ -18,13 +17,16 @@ from ...distance import Distance
 from ...epsilon import Epsilon
 from ...acceptor import Acceptor
 from ...sampler import Sampler, Sample
+from ...weighted_statistics import effective_sample_size
 from .cmd import (
     SSA, N_EVAL, N_ACC, N_REQ, N_FAIL, N_LOOKAHEAD_EVAL, ALL_ACCEPTED,
     N_WORKER, QUEUE, MSG, START, MODE, DYNAMIC, SLEEP_TIME, BATCH_SIZE,
-    IS_LOOK_AHEAD, ANALYSIS_ID, GENERATION, idfy)
+    IS_LOOK_AHEAD, ANALYSIS_ID, GENERATION, MAX_N_EVAL_LOOK_AHEAD, ACTIVE_SET,
+    idfy)
+from .util import get_active_set
 from .redis_logging import RedisSamplerLogger
 
-logger = logging.getLogger("Redis-Sampler")
+logger = logging.getLogger("ABC.Sampler")
 
 
 class RedisSamplerBase(Sampler):
@@ -155,6 +157,18 @@ class RedisEvalParallelSampler(RedisSamplerBase):
         compared to simulation, because evaluation happens sequentially on the
         main thread.
         Only effective if `look_ahead=True`.
+    max_n_eval_look_ahead_factor:
+        In delayed evaluation, only this factor times the previous number of
+        samples are generated, afterwards the workers wait.
+        Does not apply if evaluation is not delayed.
+        This allows to perform a reasonable number of evaluations only, as
+        for short-running models the number of evaluations can otherwise
+        explode unnecessarily.
+    wait_for_all_samples:
+        Whether to wait for all simulations in an iteration to finish.
+        If not, then the sampler only waits for all simulations that were
+        started prior to the last started particle of the first `n`
+        acceptances.
     log_file:
         A file for a dedicated sampler history. Updated in each iteration.
         This log file is complementary to the logging realized via the
@@ -168,12 +182,16 @@ class RedisEvalParallelSampler(RedisSamplerBase):
                  batch_size: int = 1,
                  look_ahead: bool = False,
                  look_ahead_delay_evaluation: bool = True,
+                 max_n_eval_look_ahead_factor: float = 10.,
+                 wait_for_all_samples: bool = False,
                  log_file: str = None):
         super().__init__(
             host=host, port=port, password=password, log_file=log_file)
         self.batch_size: int = batch_size
         self.look_ahead: bool = look_ahead
-        self.look_ahead_delay_evaluation = look_ahead_delay_evaluation
+        self.look_ahead_delay_evaluation: bool = look_ahead_delay_evaluation
+        self.max_n_eval_look_ahead_factor: float = max_n_eval_look_ahead_factor
+        self.wait_for_all_samples: bool = wait_for_all_samples
 
     def sample_until_n_accepted(
             self, n, simulate_one, t, *,
@@ -189,7 +207,7 @@ class RedisEvalParallelSampler(RedisSamplerBase):
             # update the SSA function
             self.redis.set(
                 idfy(SSA, ana_id, t),
-                cloudpickle.dumps((simulate_one, self.sample_factory)))
+                pickle.dumps((simulate_one, self.sample_factory)))
             # update the required population size
             self.redis.set(idfy(N_REQ, ana_id, t), n)
             # let the workers know they should update their ssa
@@ -229,16 +247,32 @@ class RedisEvalParallelSampler(RedisSamplerBase):
                     id_results.append(sample_with_id)
                     bar.update(len(id_results))
 
+        # log active set
+        _log_active_set(
+            redis=self.redis, ana_id=ana_id, t=t, id_results=id_results,
+            batch_size=self.batch_size)
+
         # maybe head-start the next generation already
         self.maybe_start_next_generation(
             t=t, n=n, id_results=id_results, all_accepted=all_accepted,
             ana_vars=ana_vars)
 
-        # wait until all workers done
-        while int(self.redis.get(idfy(N_WORKER, ana_id, t)).decode()) > 0:
-            sleep(SLEEP_TIME)
+        # wait until all relevant simulations done
+        if self.wait_for_all_samples:
+            while get_int(N_WORKER) > 0:
+                sleep(SLEEP_TIME)
+        else:
+            max_ix = max(id_result[0] for id_result in id_results)
+            while (
+                # check whether any active evaluation was started earlier
+                any(ix <= max_ix for ix in get_active_set(
+                    redis=self.redis, ana_id=ana_id, t=t))
+                # also stop if no worker is active, useful for server resets
+                and get_int(N_WORKER) > 0
+            ):
+                sleep(SLEEP_TIME)
 
-        # make sure all results are collected
+        # collect all remaining results in queue at this point
         while self.redis.llen(idfy(QUEUE, ana_id, t)) > 0:
             # pop result from queue, block until one is available
             dump = self.redis.blpop(idfy(QUEUE, ana_id, t))[1]
@@ -256,13 +290,15 @@ class RedisEvalParallelSampler(RedisSamplerBase):
                 id_results.append(sample_with_id)
 
         # set total number of evaluations
-        self.nr_evaluations_ = int(
-            self.redis.get(idfy(N_EVAL, ana_id, t)).decode())
-        n_lookahead_eval = \
-            int(self.redis.get(idfy(N_LOOKAHEAD_EVAL, ana_id, t)).decode())
+        self.nr_evaluations_ = get_int(N_EVAL)
+        n_lookahead_eval = get_int(N_LOOKAHEAD_EVAL)
 
-        # remove all time-specific variables
-        self.clear_generation_t(t)
+        # remove all time-specific variables if no more active workers,
+        #  also for previous generations
+        for _t in range(-1, t+1):
+            n_worker_b = self.redis.get(idfy(N_WORKER, ana_id, _t))
+            if n_worker_b is not None and int(n_worker_b.decode()) == 0:
+                self.clear_generation_t(t=_t)
 
         # create a single sample result, with start time correction
         sample = self.create_sample(id_results, n)
@@ -273,11 +309,15 @@ class RedisEvalParallelSampler(RedisSamplerBase):
             n_lookahead=n_lookahead_eval)
         self.logger.write()
 
+        # weight samples correctly
+        sample = self_normalize_within_subpopulations(sample, n)
+
         return sample
 
     def start_generation_t(
             self, n: int, t: int, simulate_one: Callable, all_accepted: bool,
-            is_look_ahead: bool) -> None:
+            is_look_ahead: bool, max_n_eval_look_ahead: float = np.inf,
+    ) -> None:
         """Start generation `t`."""
         ana_id = self.analysis_id
 
@@ -285,7 +325,7 @@ class RedisEvalParallelSampler(RedisSamplerBase):
         (self.redis.pipeline()
          # initialize variables for time t
          .set(idfy(SSA, ana_id, t),
-              cloudpickle.dumps((simulate_one, self.sample_factory)))
+              pickle.dumps((simulate_one, self.sample_factory)))
          .set(idfy(N_EVAL, ana_id, t), 0)
          .set(idfy(N_ACC, ana_id, t), 0)
          .set(idfy(N_REQ, ana_id, t), n)
@@ -297,7 +337,9 @@ class RedisEvalParallelSampler(RedisSamplerBase):
          .set(idfy(BATCH_SIZE, ana_id, t), self.batch_size)
          # encode as int
          .set(idfy(IS_LOOK_AHEAD, ana_id, t), int(is_look_ahead))
+         .set(idfy(MAX_N_EVAL_LOOK_AHEAD, ana_id, t), max_n_eval_look_ahead)
          .set(idfy(MODE, ana_id, t), DYNAMIC)
+         .set(idfy(ACTIVE_SET, ana_id, t), pickle.dumps(set()))
          # update the current-generation variable
          .set(idfy(GENERATION, ana_id), t)
          # execute all commands
@@ -336,7 +378,9 @@ class RedisEvalParallelSampler(RedisSamplerBase):
          .delete(idfy(N_WORKER, ana_id, t))
          .delete(idfy(BATCH_SIZE, ana_id, t))
          .delete(idfy(IS_LOOK_AHEAD, ana_id, t))
+         .delete(idfy(MAX_N_EVAL_LOOK_AHEAD, ana_id, t))
          .delete(idfy(MODE, ana_id, t))
+         .delete(idfy(ACTIVE_SET, ana_id, t))
          .delete(idfy(QUEUE, ana_id, t))
          .execute())
 
@@ -405,11 +449,23 @@ class RedisEvalParallelSampler(RedisSamplerBase):
             delay_evaluation=self.look_ahead_delay_evaluation,
             ana_vars=ana_vars)
 
+        # maximum number of look-ahead evaluations
+        if self.look_ahead_delay_evaluation:
+            # set maximum evaluations to previous simulations * const
+            nr_evaluations_ = int(
+                self.redis.get(idfy(N_EVAL, self.analysis_id, t)).decode())
+            max_n_eval_look_ahead = \
+                nr_evaluations_ * self.max_n_eval_look_ahead_factor
+        else:
+            # no maximum necessary as samples are directly evaluated
+            max_n_eval_look_ahead = np.inf
+
         # head-start the next generation
         #  all_accepted is most certainly False for t>0
         self.start_generation_t(
             n=n, t=t+1, simulate_one=simulate_one_prel,
-            all_accepted=False, is_look_ahead=True)
+            all_accepted=False, is_look_ahead=True,
+            max_n_eval_look_ahead=max_n_eval_look_ahead)
 
     def create_sample(self, id_results: List[Tuple], n: int) -> Sample:
         """Create a single sample result.
@@ -458,7 +514,7 @@ class RedisEvalParallelSampler(RedisSamplerBase):
 def create_preliminary_simulate_one(
         t, population, delay_evaluation: bool, ana_vars: AnalysisVars,
 ) -> Callable:
-    """Create a preliminary simulate_one function for generation `t+1`.
+    """Create a preliminary simulate_one function for generation `t`.
 
     Based on preliminary results, update transitions, distance function,
     epsilon threshold etc., and return a function that samples parameters,
@@ -492,11 +548,10 @@ def create_preliminary_simulate_one(
         model_perturbation_kernel=ana_vars.model_perturbation_kernel,
         transitions=transitions, model_prior=ana_vars.model_prior,
         parameter_priors=ana_vars.parameter_priors,
-        nr_samples_per_parameter=ana_vars.nr_samples_per_parameter,
         models=ana_vars.models, summary_statistics=ana_vars.summary_statistics,
         x_0=ana_vars.x_0, distance_function=ana_vars.distance_function,
         eps=ana_vars.eps, acceptor=ana_vars.acceptor,
-        evaluate=not delay_evaluation,
+        evaluate=not delay_evaluation, proposal_id=-1,
     )
 
 
@@ -561,3 +616,67 @@ def post_check_acceptance(sample_with_id, ana_id, t, redis, ana_vars,
 
     return (sample_with_id,
             any(particle.accepted for particle in sample.particles))
+
+
+def self_normalize_within_subpopulations(sample: Sample, n: int) -> Sample:
+    """Applies subpopulation-wise self-normalization of samples, in-place.
+
+    Parameters
+    ----------
+    sample: The population to be returned by the sampler.
+    n: Population size.
+
+    Returns
+    -------
+    sample: The same, weight-adjusted sample.
+    """
+    prop_ids = {particle.proposal_id for particle in sample.particles}
+
+    if len(prop_ids) == 1:
+        # Nothing to be done, as we only have one proposal, and normalization
+        #  is applied later when the population is created
+        return sample
+
+    accepted_particles = sample.accepted_particles
+    if len(accepted_particles) != n:
+        # this should not happen
+        raise AssertionError("Unexpected number of acceptances")
+
+    # get particles per proposal
+    particles_per_prop = {
+        prop_id: [particle for particle in accepted_particles
+                  if particle.proposal_id == prop_id]
+        for prop_id in prop_ids}
+
+    # normalize weights by ESS_l / sum_l[w_l] for proposal id l
+    # this is s.t. sum_i w_{l,i} \propto ESS_l
+    normalizations = {}
+    for prop_id, particles_for_prop in particles_per_prop.items():
+        weights = np.array(
+            [particle.weight for particle in particles_for_prop])
+        ess = effective_sample_size(weights)
+        total_weight = weights.sum()
+        normalizations[prop_id] = ess / total_weight
+
+    # normalize every particle (this includes rejected ones, which should not
+    #  be necessary, but does not hurt)
+    for particle in sample.particles:
+        particle.weight *= normalizations[particle.proposal_id]
+
+    return sample
+
+
+def _log_active_set(
+    redis: StrictRedis, ana_id: str, t: int, id_results: List[Tuple],
+    batch_size: int,
+) -> None:
+    """Log the status of active simulations after the first n acceptances."""
+    accepted_ids = [id_result[0] for id_result in id_results]
+    active_set = get_active_set(redis=redis, ana_id=ana_id, t=t)
+    # remove entries that are already accepted (runtime conditions)
+    active_set = active_set.difference(accepted_ids)
+    earlier = {ix for ix in active_set if max(accepted_ids) > ix}
+    logger.debug(
+        f"After {len(accepted_ids)} acceptances, "
+        f"{len(active_set) * batch_size} simulations busy, "
+        f"thereof {len(earlier) * batch_size} earlier.")
