@@ -20,7 +20,3 @@ from .execution_contexts import (DefaultContext,
                                  ProfilingContext, NamedPrinter)
 from .sge import SGE
 from .util import sge_available
-
-__all__ = ["SGE", "sge_available", "nr_cores_available",
-           "DefaultContext", "ProfilingContext",
-           "NamedPrinter"]
