@@ -44,9 +44,11 @@ class SQLiteJobDB:
         Return true if we should still wait for the job.
         Return false otherwise
         """
+        # TODO Possible SQL injection error should be fixed, e.g. via
+        #  pre-calculated expressions
         with self.connection:
             results = self.connection.execute(
-                "SELECT status, time from status WHERE ID="
+                "SELECT status, time from status WHERE ID="  # noqa: S608, B608
                 + str(ID)).fetchall()
             nr_rows = len(results)
 
