@@ -1,9 +1,8 @@
 """
-Visualizations
---------------
-
-Helper functions to visualize results of ABCSMC runs.
+Visualization
+=============
 
+Visualize results of ABCSMC runs.
 """
 
 from .kde import (
